{% load bootstrap_tags %}
<div id="waitinglist">
    <p>Enter your email address to reserve your spot in our private beta&hellip;</p>
    
    <form method="POST" action="{% url 'waitinglist_ajax_list_signup' %}" class="form-inline ajax" data-replace="#waitinglist">
        {% csrf_token %}
        {{ form|as_bootstrap }}
        <input type="submit" class="btn" value="Submit" />
    </form>
</div>